#!usr/bin/env python
# -*- coding: utf8 -*-

# -----------------------------------------------------------------------------
#   File: prepare_data_set.py (as part of project URUMETRICS)
#   Created: 20/05/2022 15:48
#   Last Modified: 20/05/2022 15:48
# -----------------------------------------------------------------------------
#   Author: William N. Havard
#           Postdoctoral Researcher
#
#   Mail  : william.havard@ens.fr / william.havard@gmail.com
#  
#   Institution: ENS / Laboratoire de Sciences Cognitives et Psycholinguistique
#
# ------------------------------------------------------------------------------
#   Description: 
#       • This script creates all the necessary directories to drop and
#         process the annotation files.
# -----------------------------------------------------------------------------

import logging
import os
import shutil

from consts import ANNOTATION_TYPES

logger = logging.getLogger(__name__)


def create_child_project_directories(project_path, message_definition):
    """
    Creates all the directories required by ChildProject
    :param project_path: path where the directories should be created
    :type project_path: str
    :return: None
    :rtype: None
    """
    child_project_dirs = [os.path.join(project_path, 'extra', 'messages', 'definition'),
                          os.path.join(project_path, 'metadata'),
                          os.path.join(project_path, 'annotations'),
                          os.path.join(project_path, 'recordings', 'raw')]

    annotations_dir = [os.path.join(project_path, 'annotations', ann_type.value[0], 'raw')
                       for ann_type in ANNOTATION_TYPES]

    all_dirs = child_project_dirs + annotations_dir

    for p in all_dirs:
        git_keep = os.path.join(p, '.gitkeep')
        if not os.path.exists(p):
            os.makedirs(p)
            open(git_keep, 'a').close()
        else:
            logging.info('{} already exists.'.format(p))

    if message_definition:
        source_file = message_definition
        target_file = os.path.join(project_path, 'extra', 'messages', 'definition', 'metrics_messages.yaml')

        assert os.path.exists(source_file), \
            ValueError('Message definition file ({}) not found!'.format(source_file))
        assert source_file.endswith('yaml'), \
            ValueError('Message definition file ({}) should end with `.yaml`'.format(source_file))
        assert not os.path.exists(target_file), \
            ValueError('A message definition file already exists! Delete it first!')

        shutil.copyfile(source_file, target_file)


def main(**kwargs):
    create_child_project_directories(**kwargs)


def _parse_args(argv):
    import argparse

    parser = argparse.ArgumentParser(description='Create a ChildProject data set.')
    parser.add_argument('--project-path', required=False, type=str, default='',
                        help="Path to a ChildProject/datalad project (useful for debugging purposes).")
    parser.add_argument('--message-definition', required=False, type=str, default='',
                        help="Path to the file containing message definitions.")
    args = parser.parse_args(argv)

    return vars(args)


if __name__ == '__main__':
    import sys

    pgrm_name, argv = sys.argv[0], sys.argv[1:]
    args = _parse_args(argv)

    logging.basicConfig(level=logging.INFO)

    try:
        main(**args)
        sys.exit(0)
    except Exception as e:
        print(e)
        sys.exit(1)
